import { ParseOptions, Node as ProseMirrorNode, Schema } from 'prosemirror-model';
import { EditorState } from 'prosemirror-state';
import { Command } from './types/command';
/**
 * Trims the whitespace around the provided block nodes.
 *
 * @param html - Input HTML content
 * @param trimAroundTags - Block elements to which trimming will be applied.
 * Defaults to block nodes of the current default schema:
 * 'div', 'ol', 'ul', 'li', 'table', 'tbody', 'thead', 'tfoot', 'td', 'th', 'p'
 * and additional table and semantic nodes from the default Angular Editor schema:
 * 'tr', 'col', 'colgroup', 'article', 'main', 'nav', 'header', 'footer', 'aside', 'section'
 *
 * @returns The trimmed HTML content
 */
export declare const trimWhitespace: (html: string, trimAroundTags?: string[]) => string;
/**
 * Creates a DocumentFragment from the given HTML content.
 *
 * @param html
 * @returns DocumentFragment
 */
export declare const htmlToFragment: (html: string) => DocumentFragment;
/**
 * @hidden
 */
export declare const fragmentToHtml: (fragment: DocumentFragment) => string;
/**
 * Creates a DocumentFragment from the given ProseMirrorNode.
 *
 * @param doc ProseMirrorNode
 * @returns DocumentFragment
 */
export declare const pmDocToFragment: (doc: ProseMirrorNode) => DocumentFragment;
/**
 * Creates a ProseMirrorNode from the given DOM element.
 *
 * @param dom
 * @param schema
 * @param parseOptions
 * @returns ProseMirrorNode
 */
export declare const domToPmDoc: (dom: Node, schema: Schema, parseOptions: ParseOptions) => ProseMirrorNode;
/**
 * Creates a ProseMirrorNode from the given HTML content.
 *
 * @param content - The new HTML content.
 * @param schema - The document schema.
 * @param parseOptions - ProseMirror parse options recognized by the `parse` and `parseSlice` methods.
 * @returns - New ProseMirrorNode instance.
 */
export declare const parseContent: (content: string, schema: Schema, parseOptions?: ParseOptions) => ProseMirrorNode;
/**
 * A function that serializes the Editor State content as HTML string.
 *
 * @param state - The Editor State
 * @returns - The serialized content
 */
export declare const getHtml: (state: EditorState | {
    doc: ProseMirrorNode;
}) => string;
/**
 * Replaces the content of the editor with a new one.
 *
 * @param content - The new HTML content.
 * @param commandName - The name of the command.
 * @param parseOptions - ProseMirror parse options recognized by the `parse` and `parseSlice` methods.
 * @returns - Command function that takes an editor `state` and `dispatch` function.
 */
export declare const setHtml: (content: string, command?: string, parseOptions?: ParseOptions) => Command;
/**
 * @hidden
 */
export declare const hasSameMarkup: (dom1: string, dom2: string, schema: Schema, parseOptions?: ParseOptions) => boolean;
/**
 * Removes comments in HTML.
 */
export declare const removeComments: (html: string) => string;
